/*********************************************************************
  All ADuC7023 library code provided by ADI, including this file, is provided
 as is without warranty of any kind, either expressed or implied.
 You assume any and all risk from the use of this code.  It is the
 responsibility of the person integrating this code into an application
 to ensure that the resulting application performs as required and is safe.

 Author        : ADI - Apps            www.analog.com/ADuC7023

 Date          : February 2009

 File          : PWM.c

 Hardware      : Applicable to ADuC7023 rev A later silicon
                 
 Description   : Configures PWM output signals
		
*********************************************************************/
#include <aduc7023.h>
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000
#define BIT17 0x20000
#define BIT20 0x100000
#define BIT21 0x200000
#define BIT24 0x1000000
void ConfigurePWM(void);

unsigned char bTripDetected = 0;
int main(void) 
{
	POWKEY1 = 0x01;						// Configure CPU Clock for 41.78MHz, CD=0
 	POWCON = 0x00;
 	POWKEY2 = 0xF4;	
	  /*
	 POWKEY3 = 0x76;			 		// Enable PWM, SPI, I2C0, I2C1 Peripherals with 41.78MHz clock
     POWCON1 = 0x0;					
	 POWKEY4 = 0xB1;
	*/
	 POWKEY3 = 0x76;			 		// Enable PWM, SPI, I2C0, I2C1 Peripherals with 41.78MHz clock
     POWCON1 = 0x924;					
	 POWKEY4 = 0xB1;
	
	GP0DAT |= 0x80000000;				// Set P0.7 as an output to turn on LED
	DAC0CON = 0x13;						// Set DAC0 output to 0-AVDD range and turn DAC0 on
	DAC0DAT = 0x04000000;				// start from midscale
	// Initialize the GPIO pins for PWM mode
	GP1CON	|= 0x22200022;  			// Select PWM0,PWM1,PWM2,PWM3 output, Trip input
	GP2CON  |= 0x202;				 	// Select PWMSYNC and PWM4 functions
	
	PWMCON	= BIT0 + BIT10; 			// PWM Standard mode, UCLK/2, PWM Trip interrupt enabled	
	
	ConfigurePWM();						// Call function to configure PWM
	bTripDetected = 0;
	IRQEN	= BIT21;	 				// Enable PWM Trip interrupt source
	while (1) 
	{
		GP0DAT ^= 0x00800000;			// Complement P0.7
	   	if (bTripDetected == 1)
	    {
	   		bTripDetected = 0;
	   		ConfigurePWM();
		}
	}				
}

void IRQ_Handler(void) __irq  			// Unused in this application
{
   unsigned long IRQSTATUS = 0;

	IRQSTATUS = IRQSTA;	   				// Read off IRQSTA register
	if ((IRQSTATUS & BIT21) == BIT21)	//If PWM trip interrupt source
	{
		 bTripDetected = 1;				// Set flag to re configure the PWM block
		 PWMCLRI = 0xFF;				// Clear the PWM trip interrupt source
	}
}
void ConfigurePWM(void)
{
   //Configure PWM0 for 20khz duty cycle
   // P1.0
	PWM0COM0 = 0x40;					//	Configure PWM0 output high trigger time
	PWM0COM1 = 0x9F;					//	Configure PWM0 output Low trigger time
	PWM0COM2 = 0x40;					//	Configure PWM1 output low trigger time
	PWM0LEN = 0xCF;						//	Configure PWM1 output high trigger time

	//Configure PWM2 for 10khz duty cycle
	// P1.6
	PWM1COM0 = 0xFF;					//	Configure PWM2 output high trigger time
	PWM1COM1 = 0x1FF;					//	Configure PWM2 output Low trigger time
	PWM1COM2 = 0xFF;					//	Configure PWM3 output low trigger time
	PWM1LEN = 0x1FF;					//	Configure PWM3 output high trigger time

	//Configure PWM4 for 40khz duty cycle
	// P2.0
	PWM2COM0 = 0x40;					//	Configure PWM4 output high trigger time
	PWM2COM1 = 0x80;					//	Configure PWM4 output Low trigger time
	PWM2COM2 = 0x40;					//	Configure PWM5 output low trigger time
	PWM2LEN = 0x80;						//	Configure PWM6 output high trigger time
}


